//! This file is generated code. Please edit the generator in
//! dev-tools/gen-target-info if you need to make changes, or see
//! src/target/llvm.rs if you need to configure a specific LLVM triple.

#[rustfmt::skip]
pub(crate) const LLVM_TARGETS: &[(&str, &str)] = &[
    ("aarch64-apple-darwin", "arm64-apple-macosx"),
    ("aarch64-apple-ios", "arm64-apple-ios"),
    ("aarch64-apple-ios-macabi", "arm64-apple-ios-macabi"),
    ("aarch64-apple-ios-sim", "arm64-apple-ios-simulator"),
    ("aarch64-apple-tvos", "arm64-apple-tvos"),
    ("aarch64-apple-tvos-sim", "arm64-apple-tvos-simulator"),
    ("aarch64-apple-visionos", "arm64-apple-xros"),
    ("aarch64-apple-visionos-sim", "arm64-apple-xros-simulator"),
    ("aarch64-apple-watchos", "arm64-apple-watchos"),
    ("aarch64-apple-watchos-sim", "arm64-apple-watchos-simulator"),
    ("aarch64-fuchsia", "aarch64-fuchsia"),
    ("aarch64-kmc-solid_asp3", "aarch64-unknown-none"),
    ("aarch64-linux-android", "aarch64-linux-android"),
    ("aarch64-nintendo-switch-freestanding", "aarch64-unknown-none"),
    ("aarch64-pc-windows-gnullvm", "aarch64-pc-windows-gnu"),
    ("aarch64-pc-windows-msvc", "aarch64-pc-windows-msvc"),
    ("aarch64-unknown-freebsd", "aarch64-unknown-freebsd"),
    ("aarch64-unknown-fuchsia", "aarch64-unknown-fuchsia"),
    ("aarch64-unknown-helenos", "aarch64-unknown-helenos"),
    ("aarch64-unknown-hermit", "aarch64-unknown-hermit"),
    ("aarch64-unknown-illumos", "aarch64-unknown-solaris2.11"),
    ("aarch64-unknown-linux-gnu", "aarch64-unknown-linux-gnu"),
    ("aarch64-unknown-linux-gnu_ilp32", "aarch64-unknown-linux-gnu_ilp32"),
    ("aarch64-unknown-linux-musl", "aarch64-unknown-linux-musl"),
    ("aarch64-unknown-linux-ohos", "aarch64-unknown-linux-ohos"),
    ("aarch64-unknown-managarm-mlibc", "aarch64-unknown-managarm-mlibc"),
    ("aarch64-unknown-netbsd", "aarch64-unknown-netbsd"),
    ("aarch64-unknown-none", "aarch64-unknown-none"),
    ("aarch64-unknown-none-softfloat", "aarch64-unknown-none"),
    ("aarch64-unknown-nto-qnx700", "aarch64-unknown-unknown"),
    ("aarch64-unknown-nto-qnx710", "aarch64-unknown-unknown"),
    ("aarch64-unknown-nto-qnx710_iosock", "aarch64-unknown-unknown"),
    ("aarch64-unknown-nto-qnx800", "aarch64-unknown-unknown"),
    ("aarch64-unknown-nuttx", "aarch64-unknown-none"),
    ("aarch64-unknown-openbsd", "aarch64-unknown-openbsd"),
    ("aarch64-unknown-redox", "aarch64-unknown-redox"),
    ("aarch64-unknown-teeos", "aarch64-unknown-none"),
    ("aarch64-unknown-trusty", "aarch64-unknown-unknown-musl"),
    ("aarch64-unknown-uefi", "aarch64-unknown-windows"),
    ("aarch64-uwp-windows-msvc", "aarch64-pc-windows-msvc"),
    ("aarch64-wrs-vxworks", "aarch64-unknown-linux-gnu"),
    ("aarch64_be-unknown-hermit", "aarch64_be-unknown-hermit"),
    ("aarch64_be-unknown-linux-gnu", "aarch64_be-unknown-linux-gnu"),
    ("aarch64_be-unknown-linux-gnu_ilp32", "aarch64_be-unknown-linux-gnu_ilp32"),
    ("aarch64_be-unknown-linux-musl", "aarch64_be-unknown-linux-musl"),
    ("aarch64_be-unknown-netbsd", "aarch64_be-unknown-netbsd"),
    ("aarch64_be-unknown-none-softfloat", "aarch64_be-unknown-none"),
    ("aarch64v8r-unknown-none", "aarch64-unknown-none"),
    ("aarch64v8r-unknown-none-softfloat", "aarch64-unknown-none"),
    ("amdgcn-amd-amdhsa", "amdgcn-amd-amdhsa"),
    ("arm-linux-androideabi", "arm-linux-androideabi"),
    ("arm-unknown-linux-gnueabi", "arm-unknown-linux-gnueabi"),
    ("arm-unknown-linux-gnueabihf", "arm-unknown-linux-gnueabihf"),
    ("arm-unknown-linux-musleabi", "arm-unknown-linux-musleabi"),
    ("arm-unknown-linux-musleabihf", "arm-unknown-linux-musleabihf"),
    ("arm64_32-apple-watchos", "arm64_32-apple-watchos"),
    ("arm64e-apple-darwin", "arm64e-apple-macosx"),
    ("arm64e-apple-ios", "arm64e-apple-ios"),
    ("arm64e-apple-tvos", "arm64e-apple-tvos"),
    ("arm64ec-pc-windows-msvc", "arm64ec-pc-windows-msvc"),
    ("armeb-unknown-linux-gnueabi", "armeb-unknown-linux-gnueabi"),
    ("armebv7r-none-eabi", "armebv7r-none-eabi"),
    ("armebv7r-none-eabihf", "armebv7r-none-eabihf"),
    ("armv4t-none-eabi", "armv4t-none-eabi"),
    ("armv4t-unknown-linux-gnueabi", "armv4t-unknown-linux-gnueabi"),
    ("armv5te-none-eabi", "armv5te-none-eabi"),
    ("armv5te-unknown-linux-gnueabi", "armv5te-unknown-linux-gnueabi"),
    ("armv5te-unknown-linux-musleabi", "armv5te-unknown-linux-musleabi"),
    ("armv5te-unknown-linux-uclibceabi", "armv5te-unknown-linux-gnueabi"),
    ("armv6-none-eabi", "armv6-none-eabi"),
    ("armv6-none-eabihf", "armv6-none-eabihf"),
    ("armv6-unknown-freebsd", "armv6-unknown-freebsd-gnueabihf"),
    ("armv6-unknown-netbsd-eabihf", "armv6-unknown-netbsdelf-eabihf"),
    ("armv6k-nintendo-3ds", "armv6k-none-eabihf"),
    ("armv7-apple-ios", "armv7-apple-ios7.0.0"),
    ("armv7-linux-androideabi", "armv7-none-linux-android"),
    ("armv7-rtems-eabihf", "armv7-unknown-none-eabihf"),
    ("armv7-sony-vita-newlibeabihf", "thumbv7a-sony-vita-eabihf"),
    ("armv7-unknown-freebsd", "armv7-unknown-freebsd-gnueabihf"),
    ("armv7-unknown-linux-gnueabi", "armv7-unknown-linux-gnueabi"),
    ("armv7-unknown-linux-gnueabihf", "armv7-unknown-linux-gnueabihf"),
    ("armv7-unknown-linux-musleabi", "armv7-unknown-linux-musleabi"),
    ("armv7-unknown-linux-musleabihf", "armv7-unknown-linux-musleabihf"),
    ("armv7-unknown-linux-ohos", "armv7-unknown-linux-ohos"),
    ("armv7-unknown-linux-uclibceabi", "armv7-unknown-linux-gnueabi"),
    ("armv7-unknown-linux-uclibceabihf", "armv7-unknown-linux-gnueabihf"),
    ("armv7-unknown-netbsd-eabihf", "armv7-unknown-netbsdelf-eabihf"),
    ("armv7-unknown-trusty", "armv7-unknown-unknown-gnueabi"),
    ("armv7-wrs-vxworks-eabihf", "armv7-unknown-linux-gnueabihf"),
    ("armv7a-kmc-solid_asp3-eabi", "armv7a-none-eabi"),
    ("armv7a-kmc-solid_asp3-eabihf", "armv7a-none-eabihf"),
    ("armv7a-none-eabi", "armv7a-none-eabi"),
    ("armv7a-none-eabihf", "armv7a-none-eabihf"),
    ("armv7a-nuttx-eabi", "armv7a-none-eabi"),
    ("armv7a-nuttx-eabihf", "armv7a-none-eabihf"),
    ("armv7a-vex-v5", "armv7a-none-eabihf"),
    ("armv7k-apple-watchos", "armv7k-apple-watchos"),
    ("armv7r-none-eabi", "armv7r-none-eabi"),
    ("armv7r-none-eabihf", "armv7r-none-eabihf"),
    ("armv7s-apple-ios", "armv7s-apple-ios"),
    ("armv8r-none-eabihf", "armv8r-none-eabihf"),
    ("asmjs-unknown-emscripten", "wasm32-unknown-emscripten"),
    ("avr-none", "avr-unknown-unknown"),
    ("avr-unknown-gnu-atmega328", "avr-unknown-unknown"),
    ("bpfeb-unknown-none", "bpfeb"),
    ("bpfel-unknown-none", "bpfel"),
    ("csky-unknown-linux-gnuabiv2", "csky-unknown-linux-gnuabiv2"),
    ("csky-unknown-linux-gnuabiv2hf", "csky-unknown-linux-gnuabiv2"),
    ("hexagon-unknown-linux-musl", "hexagon-unknown-linux-musl"),
    ("hexagon-unknown-none-elf", "hexagon-unknown-none-elf"),
    ("hexagon-unknown-qurt", "hexagon-unknown-elf"),
    ("i386-apple-ios", "i386-apple-ios-simulator"),
    ("i586-pc-windows-msvc", "i586-pc-windows-msvc"),
    ("i586-unknown-linux-gnu", "i586-unknown-linux-gnu"),
    ("i586-unknown-linux-musl", "i586-unknown-linux-musl"),
    ("i586-unknown-netbsd", "i586-unknown-netbsdelf"),
    ("i586-unknown-redox", "i586-unknown-redox"),
    ("i686-apple-darwin", "i686-apple-macosx"),
    ("i686-linux-android", "i686-linux-android"),
    ("i686-pc-nto-qnx700", "i586-pc-unknown"),
    ("i686-pc-windows-gnu", "i686-pc-windows-gnu"),
    ("i686-pc-windows-gnullvm", "i686-pc-windows-gnu"),
    ("i686-pc-windows-msvc", "i686-pc-windows-msvc"),
    ("i686-unknown-freebsd", "i686-unknown-freebsd"),
    ("i686-unknown-haiku", "i686-unknown-haiku"),
    ("i686-unknown-helenos", "i686-unknown-helenos"),
    ("i686-unknown-hurd-gnu", "i686-unknown-hurd-gnu"),
    ("i686-unknown-linux-gnu", "i686-unknown-linux-gnu"),
    ("i686-unknown-linux-musl", "i686-unknown-linux-musl"),
    ("i686-unknown-netbsd", "i686-unknown-netbsdelf"),
    ("i686-unknown-openbsd", "i686-unknown-openbsd"),
    ("i686-unknown-uefi", "i686-unknown-windows-gnu"),
    ("i686-uwp-windows-gnu", "i686-pc-windows-gnu"),
    ("i686-uwp-windows-msvc", "i686-pc-windows-msvc"),
    ("i686-win7-windows-gnu", "i686-pc-windows-gnu"),
    ("i686-win7-windows-msvc", "i686-pc-windows-msvc"),
    ("i686-wrs-vxworks", "i686-unknown-linux-gnu"),
    ("loongarch32-unknown-none", "loongarch32-unknown-none"),
    ("loongarch32-unknown-none-softfloat", "loongarch32-unknown-none"),
    ("loongarch64-unknown-linux-gnu", "loongarch64-unknown-linux-gnu"),
    ("loongarch64-unknown-linux-musl", "loongarch64-unknown-linux-musl"),
    ("loongarch64-unknown-linux-ohos", "loongarch64-unknown-linux-ohos"),
    ("loongarch64-unknown-none", "loongarch64-unknown-none"),
    ("loongarch64-unknown-none-softfloat", "loongarch64-unknown-none"),
    ("m68k-unknown-linux-gnu", "m68k-unknown-linux-gnu"),
    ("m68k-unknown-none-elf", "m68k"),
    ("mips-mti-none-elf", "mips"),
    ("mips-unknown-linux-gnu", "mips-unknown-linux-gnu"),
    ("mips-unknown-linux-musl", "mips-unknown-linux-musl"),
    ("mips-unknown-linux-uclibc", "mips-unknown-linux-gnu"),
    ("mips64-openwrt-linux-musl", "mips64-unknown-linux-musl"),
    ("mips64-unknown-linux-gnuabi64", "mips64-unknown-linux-gnuabi64"),
    ("mips64-unknown-linux-muslabi64", "mips64-unknown-linux-musl"),
    ("mips64el-unknown-linux-gnuabi64", "mips64el-unknown-linux-gnuabi64"),
    ("mips64el-unknown-linux-muslabi64", "mips64el-unknown-linux-musl"),
    ("mipsel-mti-none-elf", "mipsel"),
    ("mipsel-sony-psp", "mipsel-sony-psp"),
    ("mipsel-sony-psx", "mipsel-sony-psx"),
    ("mipsel-unknown-linux-gnu", "mipsel-unknown-linux-gnu"),
    ("mipsel-unknown-linux-musl", "mipsel-unknown-linux-musl"),
    ("mipsel-unknown-linux-uclibc", "mipsel-unknown-linux-gnu"),
    ("mipsel-unknown-netbsd", "mipsel-unknown-netbsd"),
    ("mipsel-unknown-none", "mipsel-unknown-none"),
    ("mipsisa32r6-unknown-linux-gnu", "mipsisa32r6-unknown-linux-gnu"),
    ("mipsisa32r6el-unknown-linux-gnu", "mipsisa32r6el-unknown-linux-gnu"),
    ("mipsisa64r6-unknown-linux-gnuabi64", "mipsisa64r6-unknown-linux-gnuabi64"),
    ("mipsisa64r6el-unknown-linux-gnuabi64", "mipsisa64r6el-unknown-linux-gnuabi64"),
    ("msp430-none-elf", "msp430-none-elf"),
    ("nvptx64-nvidia-cuda", "nvptx64-nvidia-cuda"),
    ("powerpc-unknown-freebsd", "powerpc-unknown-freebsd13.0"),
    ("powerpc-unknown-helenos", "powerpc-unknown-helenos"),
    ("powerpc-unknown-linux-gnu", "powerpc-unknown-linux-gnu"),
    ("powerpc-unknown-linux-gnuspe", "powerpc-unknown-linux-gnuspe"),
    ("powerpc-unknown-linux-musl", "powerpc-unknown-linux-musl"),
    ("powerpc-unknown-linux-muslspe", "powerpc-unknown-linux-muslspe"),
    ("powerpc-unknown-netbsd", "powerpc-unknown-netbsd"),
    ("powerpc-unknown-openbsd", "powerpc-unknown-openbsd"),
    ("powerpc-wrs-vxworks", "powerpc-unknown-linux-gnu"),
    ("powerpc-wrs-vxworks-spe", "powerpc-unknown-linux-gnuspe"),
    ("powerpc64-ibm-aix", "powerpc64-ibm-aix"),
    ("powerpc64-unknown-freebsd", "powerpc64-unknown-freebsd"),
    ("powerpc64-unknown-linux-gnu", "powerpc64-unknown-linux-gnu"),
    ("powerpc64-unknown-linux-musl", "powerpc64-unknown-linux-musl"),
    ("powerpc64-unknown-openbsd", "powerpc64-unknown-openbsd"),
    ("powerpc64-wrs-vxworks", "powerpc64-unknown-linux-gnu"),
    ("powerpc64le-unknown-freebsd", "powerpc64le-unknown-freebsd"),
    ("powerpc64le-unknown-linux-gnu", "powerpc64le-unknown-linux-gnu"),
    ("powerpc64le-unknown-linux-musl", "powerpc64le-unknown-linux-musl"),
    ("riscv32-wrs-vxworks", "riscv32-unknown-linux-gnu"),
    ("riscv32e-unknown-none-elf", "riscv32"),
    ("riscv32em-unknown-none-elf", "riscv32"),
    ("riscv32emc-unknown-none-elf", "riscv32"),
    ("riscv32gc-unknown-linux-gnu", "riscv32-unknown-linux-gnu"),
    ("riscv32gc-unknown-linux-musl", "riscv32-unknown-linux-musl"),
    ("riscv32i-unknown-none-elf", "riscv32"),
    ("riscv32im-risc0-zkvm-elf", "riscv32"),
    ("riscv32im-unknown-none-elf", "riscv32"),
    ("riscv32ima-unknown-none-elf", "riscv32"),
    ("riscv32imac-esp-espidf", "riscv32"),
    ("riscv32imac-unknown-none-elf", "riscv32"),
    ("riscv32imac-unknown-nuttx-elf", "riscv32"),
    ("riscv32imac-unknown-xous-elf", "riscv32"),
    ("riscv32imafc-esp-espidf", "riscv32"),
    ("riscv32imafc-unknown-none-elf", "riscv32"),
    ("riscv32imafc-unknown-nuttx-elf", "riscv32"),
    ("riscv32imc-esp-espidf", "riscv32"),
    ("riscv32imc-unknown-none-elf", "riscv32"),
    ("riscv32imc-unknown-nuttx-elf", "riscv32"),
    ("riscv64-linux-android", "riscv64-linux-android"),
    ("riscv64-wrs-vxworks", "riscv64-unknown-linux-gnu"),
    ("riscv64a23-unknown-linux-gnu", "riscv64-unknown-linux-gnu"),
    ("riscv64gc-unknown-freebsd", "riscv64-unknown-freebsd"),
    ("riscv64gc-unknown-fuchsia", "riscv64-unknown-fuchsia"),
    ("riscv64gc-unknown-hermit", "riscv64-unknown-hermit"),
    ("riscv64gc-unknown-linux-gnu", "riscv64-unknown-linux-gnu"),
    ("riscv64gc-unknown-linux-musl", "riscv64-unknown-linux-musl"),
    ("riscv64gc-unknown-managarm-mlibc", "riscv64-unknown-managarm-mlibc"),
    ("riscv64gc-unknown-netbsd", "riscv64-unknown-netbsd"),
    ("riscv64gc-unknown-none-elf", "riscv64"),
    ("riscv64gc-unknown-nuttx-elf", "riscv64"),
    ("riscv64gc-unknown-openbsd", "riscv64-unknown-openbsd"),
    ("riscv64gc-unknown-redox", "riscv64-unknown-redox"),
    ("riscv64im-unknown-none-elf", "riscv64"),
    ("riscv64imac-unknown-none-elf", "riscv64"),
    ("riscv64imac-unknown-nuttx-elf", "riscv64"),
    ("s390x-unknown-linux-gnu", "s390x-unknown-linux-gnu"),
    ("s390x-unknown-linux-musl", "s390x-unknown-linux-musl"),
    ("sparc-unknown-linux-gnu", "sparc-unknown-linux-gnu"),
    ("sparc-unknown-none-elf", "sparc-unknown-none-elf"),
    ("sparc64-unknown-helenos", "sparc64-unknown-helenos"),
    ("sparc64-unknown-linux-gnu", "sparc64-unknown-linux-gnu"),
    ("sparc64-unknown-netbsd", "sparc64-unknown-netbsd"),
    ("sparc64-unknown-openbsd", "sparc64-unknown-openbsd"),
    ("sparcv9-sun-solaris", "sparcv9-sun-solaris"),
    ("thumbv4t-none-eabi", "thumbv4t-none-eabi"),
    ("thumbv5te-none-eabi", "thumbv5te-none-eabi"),
    ("thumbv6-none-eabi", "thumbv6-none-eabi"),
    ("thumbv6m-none-eabi", "thumbv6m-none-eabi"),
    ("thumbv6m-nuttx-eabi", "thumbv6m-none-eabi"),
    ("thumbv7a-none-eabi", "thumbv7a-none-eabi"),
    ("thumbv7a-none-eabihf", "thumbv7a-none-eabihf"),
    ("thumbv7a-nuttx-eabi", "thumbv7a-none-eabi"),
    ("thumbv7a-nuttx-eabihf", "thumbv7a-none-eabihf"),
    ("thumbv7a-pc-windows-msvc", "thumbv7a-pc-windows-msvc"),
    ("thumbv7a-uwp-windows-msvc", "thumbv7a-pc-windows-msvc"),
    ("thumbv7em-none-eabi", "thumbv7em-none-eabi"),
    ("thumbv7em-none-eabihf", "thumbv7em-none-eabihf"),
    ("thumbv7em-nuttx-eabi", "thumbv7em-none-eabi"),
    ("thumbv7em-nuttx-eabihf", "thumbv7em-none-eabihf"),
    ("thumbv7m-none-eabi", "thumbv7m-none-eabi"),
    ("thumbv7m-nuttx-eabi", "thumbv7m-none-eabi"),
    ("thumbv7neon-linux-androideabi", "armv7-none-linux-android"),
    ("thumbv7neon-unknown-linux-gnueabihf", "armv7-unknown-linux-gnueabihf"),
    ("thumbv7neon-unknown-linux-musleabihf", "armv7-unknown-linux-musleabihf"),
    ("thumbv7r-none-eabi", "thumbv7r-none-eabi"),
    ("thumbv7r-none-eabihf", "thumbv7r-none-eabihf"),
    ("thumbv8m.base-none-eabi", "thumbv8m.base-none-eabi"),
    ("thumbv8m.base-nuttx-eabi", "thumbv8m.base-none-eabi"),
    ("thumbv8m.main-none-eabi", "thumbv8m.main-none-eabi"),
    ("thumbv8m.main-none-eabihf", "thumbv8m.main-none-eabihf"),
    ("thumbv8m.main-nuttx-eabi", "thumbv8m.main-none-eabi"),
    ("thumbv8m.main-nuttx-eabihf", "thumbv8m.main-none-eabihf"),
    ("thumbv8r-none-eabihf", "thumbv8r-none-eabihf"),
    ("wasm32-unknown-emscripten", "wasm32-unknown-emscripten"),
    ("wasm32-unknown-unknown", "wasm32-unknown-unknown"),
    ("wasm32-wali-linux-musl", "wasm32-wasi"),
    ("wasm32-wasi", "wasm32-wasi"),
    ("wasm32-wasip1", "wasm32-wasip1"),
    ("wasm32-wasip1-threads", "wasm32-wasi"),
    ("wasm32-wasip2", "wasm32-wasip2"),
    ("wasm32-wasip3", "wasm32-wasip3"),
    ("wasm32v1-none", "wasm32-unknown-unknown"),
    ("wasm64-unknown-unknown", "wasm64-unknown-unknown"),
    ("x86_64-apple-darwin", "x86_64-apple-macosx"),
    ("x86_64-apple-ios", "x86_64-apple-ios-simulator"),
    ("x86_64-apple-ios-macabi", "x86_64-apple-ios-macabi"),
    ("x86_64-apple-tvos", "x86_64-apple-tvos-simulator"),
    ("x86_64-apple-watchos-sim", "x86_64-apple-watchos-simulator"),
    ("x86_64-fortanix-unknown-sgx", "x86_64-elf"),
    ("x86_64-fuchsia", "x86_64-fuchsia"),
    ("x86_64-linux-android", "x86_64-linux-android"),
    ("x86_64-lynx-lynxos178", "x86_64-unknown-unknown-gnu"),
    ("x86_64-pc-cygwin", "x86_64-pc-cygwin"),
    ("x86_64-pc-nto-qnx710", "x86_64-pc-unknown"),
    ("x86_64-pc-nto-qnx710_iosock", "x86_64-pc-unknown"),
    ("x86_64-pc-nto-qnx800", "x86_64-pc-unknown"),
    ("x86_64-pc-solaris", "x86_64-pc-solaris"),
    ("x86_64-pc-windows-gnu", "x86_64-pc-windows-gnu"),
    ("x86_64-pc-windows-gnullvm", "x86_64-pc-windows-gnu"),
    ("x86_64-pc-windows-msvc", "x86_64-pc-windows-msvc"),
    ("x86_64-sun-solaris", "x86_64-pc-solaris"),
    ("x86_64-unikraft-linux-musl", "x86_64-unknown-linux-musl"),
    ("x86_64-unknown-dragonfly", "x86_64-unknown-dragonfly"),
    ("x86_64-unknown-freebsd", "x86_64-unknown-freebsd"),
    ("x86_64-unknown-fuchsia", "x86_64-unknown-fuchsia"),
    ("x86_64-unknown-haiku", "x86_64-unknown-haiku"),
    ("x86_64-unknown-helenos", "x86_64-unknown-helenos"),
    ("x86_64-unknown-hermit", "x86_64-unknown-hermit"),
    ("x86_64-unknown-hurd-gnu", "x86_64-unknown-hurd-gnu"),
    ("x86_64-unknown-illumos", "x86_64-pc-solaris"),
    ("x86_64-unknown-l4re-uclibc", "x86_64-unknown-l4re-gnu"),
    ("x86_64-unknown-linux-gnu", "x86_64-unknown-linux-gnu"),
    ("x86_64-unknown-linux-gnuasan", "x86_64-unknown-linux-gnu"),
    ("x86_64-unknown-linux-gnux32", "x86_64-unknown-linux-gnux32"),
    ("x86_64-unknown-linux-musl", "x86_64-unknown-linux-musl"),
    ("x86_64-unknown-linux-none", "x86_64-unknown-linux-none"),
    ("x86_64-unknown-linux-ohos", "x86_64-unknown-linux-ohos"),
    ("x86_64-unknown-managarm-mlibc", "x86_64-unknown-managarm-mlibc"),
    ("x86_64-unknown-motor", "x86_64-unknown-none-elf"),
    ("x86_64-unknown-netbsd", "x86_64-unknown-netbsd"),
    ("x86_64-unknown-none", "x86_64-unknown-none-elf"),
    ("x86_64-unknown-none-linuxkernel", "x86_64-unknown-none-elf"),
    ("x86_64-unknown-openbsd", "x86_64-unknown-openbsd"),
    ("x86_64-unknown-redox", "x86_64-unknown-redox"),
    ("x86_64-unknown-trusty", "x86_64-unknown-unknown-musl"),
    ("x86_64-unknown-uefi", "x86_64-unknown-windows"),
    ("x86_64-uwp-windows-gnu", "x86_64-pc-windows-gnu"),
    ("x86_64-uwp-windows-msvc", "x86_64-pc-windows-msvc"),
    ("x86_64-win7-windows-gnu", "x86_64-pc-windows-gnu"),
    ("x86_64-win7-windows-msvc", "x86_64-pc-windows-msvc"),
    ("x86_64-wrs-vxworks", "x86_64-unknown-linux-gnu"),
    ("x86_64h-apple-darwin", "x86_64h-apple-macosx"),
    ("xtensa-esp32-espidf", "xtensa-none-elf"),
    ("xtensa-esp32-none-elf", "xtensa-none-elf"),
    ("xtensa-esp32s2-espidf", "xtensa-none-elf"),
    ("xtensa-esp32s2-none-elf", "xtensa-none-elf"),
    ("xtensa-esp32s3-espidf", "xtensa-none-elf"),
    ("xtensa-esp32s3-none-elf", "xtensa-none-elf"),
];
